@@ -468,7 +468,7 @@ FlashGotDM.prototype = {
     const jobFile = this.createJobFile(job);
     const args = (fg.inPrivate || fg.getPref("shredding")) ? ["-p"] : [];
     args.push(jobFile.path);
-    this.runNative(args, false);
+    return this.runNative(args, false);
   }
 ,
   createExecutable: function() {
@@ -803,11 +803,10 @@ FlashGotDMX.prototype.performJob = function(job, blocking) {
   
   const jobFile = this.createJobFile(job);
   jobFile.permissions = 0700;
-  if (fg.isMac) {
-    this.runNative([], blocking, FlashGotDMMac.exeFile);
-  } else {
-    this.runNative([jobFile.path], blocking, this.unixShell);
-  }
+  return !(fg.isMac 
+    ? this.runNative([], blocking, FlashGotDMMac.exeFile)
+    : this.runNative([jobFile.path], blocking, this.unixShell)
+  );
 };
 FlashGotDMX.prototype.checkExePlatform = function(exeFile) {
   return this.unixShell && exeFile;
@@ -962,8 +961,7 @@ FlashGotDMMac.prototype.createJobFile = function(job) {
   return null;
 }
 FlashGotDMMac.prototype.performJob = function(job, blocking) {
-  if (this.createJobFile(job)) 
-    this.runNative([], blocking, this.exeFile);
+  return (this.createJobFile(job)) && !this.runNative([], blocking, this.exeFile);
 };
 
 FlashGotDMMac.prototype.createJob = function(links,opType) {
@@ -1179,7 +1177,7 @@ FlashGotDMCust.prototype.winEscHack = function(s) {
 FlashGotDMCust.prototype.performJob = function(job) {
   const links = job.links;
   const exeFile = links.exeFile;
-  if (links.length < 1 || !exeFile) return;
+  if (links.length < 1 || !exeFile) return false;
   
   var esc = (fg.isWindows && this.getPref("winEscHack", true))
     ? this.winEscHack : function(s) { return s; }
@@ -1222,6 +1220,7 @@ FlashGotDMCust.prototype.performJob = function(job) {
        false, exeFile);
     this.updateProgress(links, j, len);
   }
+  return true;
 };
 FlashGotDMCust.prototype.checkExePlatform = function(exeFile) {
   return exeFile;
@@ -1416,10 +1415,13 @@ FlashGotDM.initDMS = function() {
               url: wrapURL(l.href, cs),
               description: l.description,
               ultDescription: '',
-              referrer: referrer,
-              fileName: l.fname
+              referrer: referrer
           });
           
+          if ("fname" in l) {
+            lastItem.fileName = lastItem.destinationName = l.fname;
+            // if (single && !turbo) single = false;
+          }
           if (arr == anchors && mlSupport && l.href.indexOf("#") > 0) {
             hash = l.href.match(/.*#(.*)/)[1];
             ml = mlSupport(l.href);
@@ -1443,10 +1445,13 @@ FlashGotDM.initDMS = function() {
           if (!lastItem) {
             this.log("DTA: no link found in " + links.length);
           } else if (single && (DTA.saveSingleItem || w.DTA_AddingFunctions.saveSingleItem)) {
+            this.log("DTA.saveSingleItem " + lastItem.toSource());
             DTA.saveSingleItem(w, turbo, lastItem);
           } else if (single && (DTA.saveSingleLink || w.DTA_AddingFunctions.saveSingleLink)) {
+            this.log("DTA.saveSingleLink " + lastItem.url);
             DTA.saveSingleLink(w, turbo, lastItem.url, lastItem.referrer, lastItem.description, links.postData);
           } else {
+            this.log("DTA.saveLinkArray " + anchors.toSource());
             DTA[turbo ? "turboSaveLinkArray" : "saveLinkArray"](w, anchors, images);
           }
           break;
@@ -1666,12 +1671,13 @@ FlashGotDM.initDMS = function() {
   dm.performJob = function(job) {
     const file = this.createJobFile(job);
     if (this.exeFile) {
-      this.runNative(['-f', file.path, '-b', this.browser])
+      return !this.runNative(['-f', file.path, '-b', this.browser])
     } else {
       CC["@presenta/iGetter"]
               .getService(CI.nsIGetterMoz)
               .NewURL(file.path);
       if (file.exists()) file.remove(0);
+      return true;
     }
   };
   
@@ -1731,7 +1737,7 @@ FlashGotDM.initDMS = function() {
       dm.reGetService = CC["@reget.com/regetextension;1"].createInstance(CI.IRegetDownloadFFExtension);
       if (dm.reGetService.isExtensionEnabled()) {
         dm._supported = true;
-        dm.performJob = function() {};
+        dm.performJob = function() { return true };
         dm.createJob = function(links, opType) {
           const rg = this.reGetService;
           var l;
@@ -1979,14 +1985,14 @@ FlashGotDM.initDMS = function() {
     }
     
     if (callback) {
-      r.onreadystatechange = function() {
-        if (r.readyState == 4) {
+      r.addEventListener("readystatechange", function() {
+        if (r.readyState === 4) {
           try {
             fg.log("JDownloader response:\n" + r.status + "\n" + r.responseText);
           } catch (e) {}
           callback(r);
         }
-      };
+      }, false);
     }
     return r;
   }
@@ -2140,7 +2146,7 @@ FlashGotDM.initDMS = function() {
     return FlashGotDMX.prototype.createJob.call(this, links, opType)
   } 
   
-  dm = new FlashGotDMX("KDE KGet","kget");
+  dm = new FlashGotDMX("KDE KGet","kget", "[ULIST]");
   dm.askPath = ASK_NEVER;
   
   if (isWin) {
@@ -2187,6 +2193,25 @@ FlashGotDM.initDMS = function() {
   dm = new FlashGotDMX("Wget", "wget", '-c [-O FNAME] [--directory-prefix=FOLDER] [--referer=REFERER] [--post-data=POST] [--load-cookies=CFILE] [--header=Cookie:COOKIE] [--input-file=UFILE] [--user-agent=UA] [URL]');
   dm.postSupport = true;
   dm.terminal = true;
+  dm.__defineGetter__("_trustServerOpt", function() {
+    const outFile = fg.tmpDir.clone();
+    outFile.append("wgetHelp.txt");
+    if (this.performJob(this.unixCmd + " -h >'" + outFile.path + "' 2>&1", true)) {
+      delete this._trustServerOpt;
+      return this._trustServerOpt = IO.readFile(outFile).indexOf("--trust-server-names") !== -1;
+    }
+    return false;
+  });
+  dm.createJob = function(links, opType) {
+    const homeDir = fg.directoryService.get("Home", CI.nsILocalFile).path;
+    const opt = "--trust-server-names ";
+    if (links.folder == homeDir)
+      while (this.argsTemplate.indexOf(opt) !== -1)
+        this.argsTemplate = this.argsTemplate.replace(opt, "");
+    else if (this._trustServerOpt && this.argsTemplate.indexOf(opt) < 0)
+      this.argsTemplate = opt + this.argsTemplate;
+    return this.__proto__.createJob.call(this, links, opType);
+  }
   
   if (fg.isWindows) {
     dm = new FlashGotDMCust("Uget");
@@ -2490,9 +2515,8 @@ FlashGotDM.initDMS = function() {
           persist.progressListener = dl = 
             mozAddDownload(uri, IOS.newFileURI(file), file.leafName, persist)
               .QueryInterface(CI.nsIWebProgressListener);
-          const cacheKey  = CC['@mozilla.org/supports-string;1'].createInstance(CI.nsISupportsString);
-          cacheKey.data = uri.spec;
-          persist.saveURI(uri, cacheKey,
+          
+          persist.saveURI(uri, l.cacheKey || null,
                   links.refURI, postData, null, file);
         } catch (e) {
           fg.log("Skipping link " + l.href + ": " + e);
