@@ -93,22 +93,30 @@ var MediaSniffer = {
     return m && { fname: m[1], contentType: contentType };
   },
   
+  typesWhitelistRx: /\b(?:x?html|image|css|j(?:ava(?:script)?|son)|shockwave)\b/i,
   
+  get downloadInterface() {
+    delete this.downloadInterface;
+    return this.downloadInterface = "nsIHttpActivityObserver" in CI ? CI.nsIProgressEventSink : CI.nsIWebBrowserPersist;
+  },
   
   observe: function(channel, topic, data) {
     if (channel instanceof CI.nsIChannel) {
       try {
         var contentType = channel.contentType;
-        if (!contentType || /\b(?:x?html|image|css|javascript|shockwave)\b/i.test(contentType))
+        if (!contentType || this.typesWhitelistRx.test(contentType))
           return;
-         
-        if (this.debug) dump("Examining " + channel.URI.spec + " (" + contentType + ")\n");
+        
+        if (channel.notificationCallbacks instanceof this.downloadInterface) {
+          if (this.debug) dump("Skipping " + channel.name + ": from the built-in download manager or DTA\n" + channel.notificationCallbacks + "\n\n");
+          return;
+        }
+          
+        if (this.debug) dump("Examining " + channel.name + " (" + contentType + ")\n");
         
         var typeInfo = null;
         if (this.mediaTypesRx.test(contentType) || (typeInfo = this.sniffType(channel))) {
-          
-          
-          
+
           var win = (DOM.findChannelWindow(channel) || DOM.mostRecentBrowserWindow.content).top;
 
           if (this.debug) dump("Media Window: " + win + " - " + win.location.href + " -- " + contentType + "\n");
@@ -158,12 +166,12 @@ var MediaSniffer = {
             var host = (channel.originalURI || channel.URI).host;
             if (host) {
               host = host.replace(/\./g, '_');
-              while(host && (redirect = fg.getPref("media.redirect." + host, -1)) == -1) {
+              while(host && (redirect = fg.getPref("media.redirect." + host, -1)) === -1) {
                 host = host.replace(/.*?(?:_|$)/, '');
               }
             }
-            if (redirect == -1) redirect = fg.getPref("media.redirect", 0); // 0 - no redirect, 1 redirect, 2 include both initial and final url
-            if (redirect == 0) url = channel.originalURI && channel.originalURI.spec || url;
+            if (redirect === -1) redirect = fg.getPref("media.redirect", 0); // 0 - no redirect, 1 redirect, 2 include both initial and final url
+            if (redirect === 0) url = channel.originalURI && channel.originalURI.spec || url;
             
             var size = contentLength < 0 ? "???KB"
              : contentLength < 1024 ? (contentLength + "B")
@@ -183,7 +191,8 @@ var MediaSniffer = {
                   (win.location.host || '').split(".").filter(function(s) { return s }).join("|")  + ")\\b", 'ig'), '')
                   .replace(/https?:\/{2}/gi, '')
                   .replace(unicode ? /[\u0000-\u0020\u2000-\u206F\u2E00-\u2E7F]+|[^\w\u0080-\uffff]+/g : /\W+/g, '_')
-                  .replace(/^_+([^_])|([^_])_+$/g, '$1')
+                  .replace(/^_+([^_])/g, '$1')
+                  .replace(/([^_])_+$/g, '$1')
                   .replace(/_+/g, '_')
             }
             
@@ -202,8 +211,9 @@ var MediaSniffer = {
             var description = title + " (" + contentType + ", " + size + ")";
             tip += " (" + size + ")";
             
+            var entry;
             while(url) {
-              media.push (map[url] = {
+              media.push (entry = map[url] = {
                 href: (fname && fg.getPref("media.forceNameHack", true) && url.indexOf("#") == -1) ? url + "#/" + encodeURIComponent(fname) : url,
                 referrer: (channel instanceof CI.nsIHttpChannel) && channel.referrer && channel.referrer.spec,
                 description: title + " (" + contentType + ")",
@@ -214,7 +224,16 @@ var MediaSniffer = {
                 fname: fname
               });
               
-              url = redirect == 2 && (!channel.originalURI || channel.originalURI.spec == url ? null : channel.originalURI.spec);
+              if (channel instanceof CI.nsICachingChannel) {
+                entry.cacheKey = channel.cacheKey;
+                if (fg.getPref("media.forceCacheAsFile", true)) try {
+                  channel.cacheAsFile = true;
+                } catch(e) {
+                  if (this.debug) dump("Couldn't force cacheAsFile: " + e + "\n");
+                }
+                if (this.debug) dump(entry.href + " cache as file: " + channel.cacheAsFile + ", loadFlags:" + this.humanFlags(channel.loadFlags) + "\n");
+              }
+              url = redirect === 2 && (!channel.originalURI || channel.originalURI.spec == url ? null : channel.originalURI.spec);
             }
           }
 
@@ -235,7 +254,27 @@ var MediaSniffer = {
       }
     }
   },
-    
+  
+  get _channelFlags() {
+    delete this._channelFlags;
+    const constRx = /^[A-Z_]+$/;
+    const ff = {};
+    [CI.nsIHttpChannel, CI.nsICachingChannel].forEach(function(c) {
+      for (var p in c) {
+        if (constRx.test(p)) ff[p] = c[p];
+      }
+    });
+    return this._channelFlags = ff;
+  },
+  humanFlags: function(loadFlags) {
+    var hf = [];
+    var c = this._channelFlags;
+    for (var p in c) {
+      if (loadFlags & c[p]) hf.push(p + "=" + c[p]);
+    }
+    return hf.join("\n");
+  },
+  
   limitFName: function(fname) {
     const MAX_FILE_LEN = 128, MAX_EXT_LEN = 5;
     var dotPos = fname.lastIndexOf(".");
