@@ -166,7 +166,7 @@ RedirectContext.prototype = {
     
     callbacks.call(0);
     
-    req.onreadystatechange = function() {
+    req.addEventListener("readystatechange", function() {
       var phase = req.readyState;
       try {
         callbacks.call(phase);
@@ -182,7 +182,7 @@ RedirectContext.prototype = {
           context.done();
         }
       }
-    };
+    }, false);
     
     this.start();
     try {
@@ -600,7 +600,7 @@ RedirectContext.prototype = {
         var xhr = context.createReq();
         xhr.open("GET", l.href);
         if (xhr.channel instanceof CI.nsIHttpChannel) xhr.channel.redirectionLimit = 0;
-        xhr.onreadystatechange = function() {
+        xhr.addEventListener("readystatechange", function() {
           if (xhr.readyState != 4) return;
           if (xhr.status == 200 && xhr.channel.contentType == "text/html") {
             if (!/\bdelete/.test(xhr.responseText)) {
@@ -609,7 +609,7 @@ RedirectContext.prototype = {
             }
           } else context.rapidPro = true;
           context.done();
-        }
+        }, false);
         xhr.send(null);
         context.start();
       }
