@@ -28,7 +28,7 @@ const NS_BINDING_ABORTED = 0x804b0002;
 const EXTENSION_ID = "{19503e42-ca3c-4c27-b1e2-9cdb2170ee34}";
 const EXTENSION_NAME = "FlashGot";
 const CHROME_NAME = "flashgot";
-const VERSION = "1.3.0.5";
+const VERSION = "1.3.3";
 const SERVICE_NAME = EXTENSION_NAME + " Service";
 const SERVICE_CTRID = "@maone.net/flashgot-service;1";
 const SERVICE_ID = "{2a55fc5c-7b31-4ee1-ab15-5ee2eb428cbe}";
@@ -911,8 +911,7 @@ fg = {
         }
       }
       if (this.logFile) this.logFile.remove(true);
-      this.logStream = null;
-      this.log(null);
+      this.logInit();
     } catch(ex) { dump(ex.message); }
   } 
 ,
@@ -1161,7 +1160,7 @@ fg = {
       }
       
       try {
-        FlashGotDM.cleanup(this.uninstalling);
+        if (typeof FlashGotDM === "object") FlashGotDM.cleanup(this.uninstalling);
       } catch(eexx) {
         dump(eexx.message);
       }
